from __future__ import annotations
from dataclasses import dataclass

from .base.qr_status import CheckTransactionStatus

# Responese data
@dataclass
class CheckTransactionData:
    payment_status_code: int
    total_amount: float
    original_amount: float
    refund_amount: float
    discount_amount: float
    payment_amount: float
    payment_currency: str
    apv: str
    payment_status: str
    transaction_date: str


    @classmethod
    def from_dict(cls, data: dict) -> "CheckTransactionData":
        return cls(
            payment_status_code= int(data.get("payment_status_code", -1)),
            total_amount=float(data.get("total_amount", 0)),
            original_amount=float(data.get("original_amount", 0)),
            refund_amount=float(data.get("refund_amount", 0)),
            discount_amount=float(data.get("discount_amount", 0)),
            payment_amount=float(data.get("payment_amount",0)),
            payment_currency=data.get("payment_currency", ""),
            apv=data.get("apv", ""),
            payment_status=data.get("payment_status", ""),
            transaction_date=data.get("transaction_date", ""),
        )
    @property
    def is_approved(self) -> bool:
        return self.payment_status == "APPROVED"
    @property
    def is_pending(self) -> bool:
        return self.payment_status == "PENDING"

@dataclass
class CheckTransactionResponse:
    data: CheckTransactionData
    status: CheckTransactionStatus

    @classmethod
    def from_dict(cls, raw: dict) -> "CheckTransactionResponse":
        return cls(
            data=CheckTransactionData.from_dict(raw.get("data", {})),
            status=CheckTransactionStatus.from_dict(raw.get("status", {}))
        )


from dataclasses import dataclass

@dataclass
class DataRequest:
    ''' Shared base for all request structure for Check, Close transaction apis'''
    tran_id: str

    #filled auto by the client
    req_time: str = ""
    merchant_id: str = ""
    hash: str = ""

    def to_payload(self) -> dict:
        return {
            "req_time": self.req_time,
            "merchant_id": self.merchant_id,
            "tran_id": self.tran_id,
            "hash": self.hash,
        }
    def hash_values(self) -> tuple:
        return(
            self.req_time,
            self.merchant_id,
            self.tran_id,
        )

# Create alias
CheckTransactionRequest = DataRequest
CloseTransactionRequest = DataRequest



from .base import BaseClient
from ..models.base.data_request import CheckTransactionRequest, CloseTransactionRequest
from ..models.check_transaction import CheckTransactionResponse
from ..models.close_transaction import CloseTransactionResponse

class TransactionClient(BaseClient):

    _ENDPOINT_CHECK = "api/payment-gateway/v1/payments/check-transaction-2"
    _ENDPOINT_CLOSE = "api/payment-gateway/v1/payments/close-transaction"

    def check(self, tran_id: str) -> CheckTransactionResponse:
        request = CheckTransactionRequest(tran_id=tran_id)
        return self._prepare_and_validate(self._ENDPOINT_CHECK, request, CheckTransactionResponse)

    def close(self, tran_id: str) -> CloseTransactionResponse:
        request = CloseTransactionRequest(tran_id=tran_id)
        return self._prepare_and_validate(self._ENDPOINT_CLOSE, request, CloseTransactionResponse)


CheckTransactionClient = TransactionClient

__all__ = ["TransactionClient", "CheckTransactionClient"]


from __future__ import annotations
from dataclasses import dataclass

from .base.qr_status import CloseTransactionStatus

# Response
@dataclass
class CloseTransactionResponse:
    status: CloseTransactionStatus

    @classmethod
    def from_dict(cls, raw: dict) -> "CloseTransactionResponse":
        return cls(
            status=CloseTransactionStatus.from_dict(raw.get("status", {})),
        )
    @property
    def is_cancelled(self) -> bool:

        # ABA use "00" for success
        return self.status.is_success
